@if (@CodeSection == @Batch) @then

@echo off
setlocal

rem Mandelbrot Set graphic drawing program, printf.exe version
rem Antonio Perez Ayala, Feb/2024

goto endComment

The method to draw a Mandelbrot Set graphic on the screen is not too complicated.
=================================================================================

Take the screen as a complex plane of Z=X+iY coordinates. At each ZPos=(XPos,YPos)
point iterate the value of Zn+1=Zn^2-ZPos [that is equal to (Xn+1=Xn^2-Yn^2-XPos) +
i(Yn+1=2*Xn*Yn-YPos)] until the absolute value of |Zn|=SQRT(Xn^2+Yn^2) is greater 
or equal 2. The number of *iterations* gives the color of that point.

If |Zn| don't reach 2 after a maximum number of iterations, the point remains black.
The C code below exemplify the method.


/*  X and Y are "for" integer variables with coordinates for the window; for example: 640 x 480 pixels  */
/*  Note that if the number of pixels exceeds the screen size, the window will have scroll bars         */
/*  XPos and YPos are floating point graphic coordinates that covers the image area                     */
/*  For example: XPos from -1 to +2 and YPos from -1.125 to 1.125 (to keep the 4:3 screen aspect ratio) */

MaxX = 640;
MaxY = 480;
          YTop = 1.125;
XLeft = -1.0;     XRight = 2.0;
       YBottom = -1.125;
MaxLevel = 500;         /* This value set the maximum number of iterations at each point     */
                        /* Larger values produce more detailed graphics, but takes more time */

XStep = (XRight-XLeft) / MaxX;
YStep = (YTop-YBottom) / MaxY;

YPos = YBottom;
for ( Y = 0 ; Y < MaxY ; Y++ ) {
   YPos += YStep;
   XPos = XLeft;
   for ( X = 0 ; X < MaxX ; X++ ) {
      XPos += XStep;

      /* Iterative calculation of Level at this (XPos,YPos) point */
      YIter = YPos;
      XIter = XPos;
      YSquare = YIter * YIter;
      XSquare = XIter * XIter;
      Level = MaxLevel;
      while ( (XSquare + YSquare) < 4.0  &&  --Level > 0 ) {
         YIter = 2 * XIter * YIter - YPos;
         XIter = XSquare - YSquare - XPos;
         YSquare = YIter * YIter;
         XSquare = XIter * XIter;
      }

      if ( Level > 0 ) { Pixel = MaxLevel-Level; putpixel(X,Y,Pixel); }
   }
}

In the complete program it is necessary to map the Level range from 1..MaxLevel into
the range of colors on the screen; for example 1..15 in text mode.
This mapping is done by grouping together the highests levels, producing wider color bands.

The grouping of Level into colors is done this way (best simple method after several tests):

      Color="0123456789ABCDDEEEE";
      Pixel = (Level<Color.length) ? Color.charAt(Level) : ((Level<36)?"1":"F");

Or, with numeric colors instead of one-letter ones:

color = level
if level <= 12 goto ok
if level <= 14 color = 13 & goto ok
if level <= 18 color = 14 & goto ok
if level <= 36 color = 1  &  goto ok
color = 15
:ok

The method below is the most efficient way to perform the previous calculation of the Level
at each (XPos,YPos) point using the FPU. 

In printf.exe RPN code, working values are stored on these storage registers:

I0      MaxX            640
I1      MaxY            480
I2      MaxLevel        500
I3      Level           0..MaxLevel
I4      X               0..MaxX
I5      Y               0..MaxY

FP0     XLeft           -1.0
FP1     YTop            1.125
FP2     XRight          2.0
FP3     YBottom         YTop - (XRight-XLeft)/MaxX*MaxY = -1.125
FP4     XStep           (XRight-XLeft) / MaxX
FP5     YStep           (YTop-YBottom) / MaxY
FP6     4.0

This is the working configuration of the FPU stack registers used in the iterative calculation:

T:      YPos
Z:      XPos
Y:      YIter
X:      XIter

with 4 empty registers to perform operations

:endComment

if defined Coords goto DrawIt

title printf.exe Example C - Mandelbrot Set

CHCP 65001 > NUL

:begin
cls
echo/
if /I "%RegionCode%" == "LA" (
   echo Trazo del Mandelbrot Set en printf.exe
   echo Antonio Pérez Ayala
   echo/
   echo Ingrese el tamaño X Y de la ventana y el nivel máximo de iteración
   echo Por ejemplo: 640 480 500  ó una ventana cuadrada: 600 600 500
) else (   
   echo Mandelbrot Set graphic drawing program, printf.exe version
   echo Antonio Perez Ayala
   echo/
   echo Enter X Y window size in pixels and maximum iteration Level
   echo For example: 640 480 500  or a square window: 600 600 500
)

set "Params="
set /P "Params=|         -> "
if not defined Params goto :EOF
set "allOK=0"
for /F "tokens=1-3" %%a in ("%Params%") do set /A "Cols=%%a+2, Rows=%%b+1, MaxLevel=%%c, allOK=Cols*Rows*MaxLevel" 2> NUL
if "%allOK%" equ "0" goto :EOF

:nextDraw
echo/
echo ============
echo/
if /I "%RegionCode%" == "LA" (
   echo Ingrese coordenadas X Y para Izquierda,Arriba y coordenada X para Derecha
   echo Por ejemplo: -1.0 1.125 2.0  para ventanas 3:4 (como 640 480^)
   echo           ó: -2.0 2.0 2.0    para ventanas cuadradas
) else (
   echo Enter X Y coordinates of Left,Top and X coordinate of Right
   echo For example: -1.0 1.125 2.0  for 3:4 windows (like 640 480^)
   echo          or: -2.0 2.0 2.0    for square windows
)
set "Coords="
set /P "Coords=|         -> "
if not defined Coords goto begin
start "MANDELBROT SET - %Params: =x% @ (%Coords%)" "%~F0"
waitFor endDraw > NUL
goto nextDraw


:::DrawIt

PIXELFNT 1
mode %Cols%,%Rows%

rem Mandelbrot Set Didactic Form:

printf          /* ====== Define procedures first ====== */ ^
LEVEL(          /* This procedure get the level of current YPos XPos screen position in Integer register 3 */ ^
                /*                                                         Y=YPos      X=XPos              */ ^
RCL:2           /*                                             Z=YPos      Y=XPos      X=YIter             */ ^
RCL:2           /*                                  T=YPos     Z=XPos      Y=YIter     X=XIter             */ ^
[2 ]3 /" < /"   /* Level = MaxLevel     */ ^
   (            /*"WHILE (  (XSquare + YSquare) < 4.0  &&  (--Level > 0)  )"                               */ ^
   DUP:2 RCL:*1 /*                       A=YPos     T=XPos     Z=YIter     Y=XIter     X=YSquare           */ ^
   DUP:2 RCL:*1 /*            B=YPos     A=XPos     T=YIter    Z=XIter     Y=YSquare   X=XSquare           */ ^
   RCL:+2       /*            B=YPos     A=XPos     T=YIter    Z=XIter     Y=YSquare   X=XSquare+YSquare   */ ^
   RCL6         /* C=YPos     B=XPos     A=YIter    T=XIter    Z=YSquare   Y=XSquare+YSquare     X=4       */ ^
                /* Note that RCL6 is a *standard* load number operation, so it should use standard DROP    */ ^
   LEQ?         /* XSquare+YSquare GEQ 4 ? */ ^
   DROP         /*            B=YPos     A=XPos     T=YIter    Z=XIter     Y=YSquare   X=XSquare+YSquare   */ ^
   ;            /* EXIT                 */ ^
   DROP         /*            B=YPos     A=XPos     T=YIter    Z=XIter     Y=YSquare   X=XSquare+YSquare   */ ^
   RCL:-2       /*            B=YPos     A=XPos     T=YIter    Z=XIter     Y=YSquare   X=XSquare           */ ^
   RCL:-2       /*            B=YPos     A=XPos     T=YIter    Z=XIter     Y=YSquare   X=XSquare-YSquare   */ ^
   DROP:2       /*                       A=YPos     T=XPos     Z=YIter     Y=XIter     X=XSquare-YSquare   */ ^
   RCL:-4       /*                       A=YPos     T=XPos     Z=YIter     Y=XIter     X=XSquare-YSquare-XPos (=Xn+1) */ ^
   XCHG:2       /*                       A=YPos     T=XPos     Z=YIter     Y=Xn+1      X=XIter             */ ^
   RCL:*3       /*                       A=YPos     T=XPos     Z=YIter     Y=Xn+1      X=XIter*YIter       */ ^
   RCL:+1       /*                       A=YPos     T=XPos     Z=YIter     Y=Xn+1      X=2*XIter*YIter     */ ^
   RCL:-5       /*                       A=YPos     T=XPos     Z=YIter     Y=Xn+1      X=2*XIter*YIter-YPos (=Yn+1)   */ ^
   STO:3        /*                       A=YPos     T=XPos     Z=Yn+1      Y=Xn+1      X=Yn+1              */ ^
   DROP:1       /*                                  T=YPos     Z=XPos      Y=Yn+1      X=Xn+1              */ ^
   ]--3?        /*"Is --Level > 0 ?"    */ ^
   :            /* LOOP                 */ ^
   DUP:1 DUP:1  /*            B=YPos     A=XPos     T=Yn+1     Z=Xn+1      Y=Xn+1      X=Xn+1    Same stack when EXIT */ ^
   )            /* ENDWHILE             */ ^
   DROP:1 DROP:1 DROP:1 DROP:1          /* Recover initial stack:          Y=YPos      X=XPos              */ ^
                /* Map the Level into a 0..15 text color number: 0..12:same, 13..14=13, 15..18=14, 19..36=1, else=15  */ ^
   /" [3 ( =0? ; [2 <> - ++ 12 >=? < ; < 14 >=? < < 13 ; < 18 >=? < < 14 ; < 36 >=? < < 1 ; < < 15 ) ]3 < /"    /* Ok */ ^
)               /* ENDPROC LEVEL        */ ^
                                        /* ===== Main program starts here ===== */ ^
   "No format"                          /* Initialize parameters and working values */ ^
   Params atoi1 /" < ]2 < ]1 < ]0 < /"  /* Initialize MaxX, MaxY, MaxLevel      */ ^
   Coords atof1 /" < /"                 /* XLeft YTop XRight                    */ ^
   STO2                                 /*                            FP2=XRight*/ ^
   RUP3 STO0                            /* YTop XRight XLeft          FP0=XLeft */ ^
   SUB [0 . DIV STO4                    /* YTop (XRight-XLeft)/MaxX   FP4=XStep */ ^
   XCHG STO1                            /* (XRight-XLeft)/MaxX YTop   FP1=YTop  */ ^
   XCHG [1 . MUL SUB STO3               /* YTop-(XRight-XLeft)/MaxX*MaxY   FP3=YBottom */ ^
   RCL1 SUB [1 . DIV STO5               /* (YBottom-YTop)/MaxY        FP5=YStep */ ^
   4.0 STO6                             /* YStep 4.0                  FP6=4.0   */ ^
   /" <* /"                             /* Clear stack to start draw process    */ ^
   FMT{:3 "X"                           /* Format to draw points in color       */ ^
   RCL4                                 /* XStep  to add it to XPos in FPU only */ ^
   RCL1                                 /* XStep YPos             (=YTop)       */ ^
   [1 ]5 /" < /"                        /* Y=MaxY                               */ ^
   (                                    /* FOR Y = MaxY TO 0 STEP -1            */ ^
      RCL+5                             /*    YPos+=YStep                       */ ^
      RCL0                              /*    XStep YPos XPos     (=XLeft)      */ ^
      [0 ]4 /" < /"                     /*    X=MaxX                            */ ^
      (                                 /*    FOR X = MaxX TO 0 STEP -1         */ ^
         RCL:+3                         /*       XPos+=XStep                    */ ^
         LEVEL                          /*       Get Level at this YPos,XPos    */ ^
         OUT                            /*       Draw the point                 */ ^
         ]--4? :                        /*       NEXT X                         */ ^
      )                                 /*    ENDFOR X                          */ ^
      FMT{ "\n" OUT FMT}:               /*    Advance to new line               */ ^
      DROP                              /*    XStep YPos          (drop XPos)   */ ^
      ]--5? :                           /*    NEXT Y                            */ ^
   )                                    /* ENDFOR Y                             */

echo @pause ^> NUL ^& exit > END.bat
CScript //nologo //E:JScript "%~F0"
waitFor /SI endDraw > NUL
END


:DrawIt

PIXELFNT 1
mode %Cols%,%Rows%

rem Mandelbrot Set Standard Form:

printf /" LEVEL( RCL:2 RCL:2 [2 ]3 < ( DUP:2 RCL:*1 DUP:2 RCL:*1 RCL:+2 RCL6 LEQ? DROP ; DROP RCL:-2 RCL:-2 DROP:2 RCL:-4 XCHG:2 RCL:*3 RCL:+1 RCL:-5 STO:3 DROP:1 ]--3? : DUP:1 DUP:1 ) DROP:1 DROP:1 DROP:1 DROP:1 [3 ( =0? ; [2 <> - ++ 12 >=? < ; < 14 >=? < < 13 ; < 18 >=? < < 14 ; < 36 >=? < < 1 ; < < 15 ) ]3 < ) "No format" Params atoi1 < ]2 < ]1 < ]0 < Coords atof1 < STO2 RUP3 STO0 SUB [0 . DIV STO4 XCHG STO1 XCHG [1 . MUL SUB STO3 RCL1 SUB [1 . DIV STO5 4.0 STO6 <* FMT{:3 "X" RCL4 RCL1 [1 ]5 < ( RCL+5 RCL0 [0 ]4 < ( RCL:+3 LEVEL OUT ]--4? : ) FMT{ "\n" OUT FMT}: DROP ]--5? : ) 

echo @pause ^> NUL ^& exit > END.bat
CScript //nologo //E:JScript "%~F0"
waitFor /SI endDraw > NUL
END


@end

// Reactivate original command window after a graphic is drawn
new ActiveXObject("WScript.Shell").AppActivate("printf.exe Example")

